<?php

declare(strict_types=1);

namespace Drupal\dofacshs\Hook;

use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\Core\Hook\Attribute\Hook;

/**
 * Hook implementations for .
 */
class EntitySave {

  /**
   * Implements hook_entity_presave().
   */
  #[Hook('entity_presave')]
  public function autoSetFilterTerms(EntityInterface $entity) {
    if (! $entity instanceof ContentEntityInterface) {
      return;
    }
    $vocab_reference = \Drupal::config('dofacshs.settings')->get('vocabulary_reference') ?? '';
    [$display_vocab, $filter_field, $filter_vocab] = explode('.', $vocab_reference);
    $source = \Drupal::config('dofacshs.settings')->get('entity_source_field') ?? '';
    [$entity_type, $bundle, $source_field, $source_target_bundle] = explode('.', $source);
    $target = \Drupal::config('dofacshs.settings')->get('entity_target_field') ?? '';
    [$tentity_type, $tbundle, $target_field, $target_target_bundle] = explode('.', $target);
    if ($entity->getEntityTypeId() !== $entity_type || $entity->bundle() !== $bundle) {
      // We are not meant to act on this entity, bail quickly.
      return;
    }
    // Quick validation.  @TODO put this validation on the form side too!!
    if (!$display_vocab || !$entity_type || !$bundle || !$filter_field || !$filter_vocab || !$source_field || !$target_field || $display_vocab !== $source_target_bundle || $filter_vocab !== $target_target_bundle || $entity_type !== $tentity_type || $bundle !== $tbundle) {
      \Drupal::logger('dofacshs')->warning('Invalid configuration for Display One Filter Another with Client-side Hierarchical Select.');
      return;
    }
    // Remove duplicate vars used to validate (or get) the configuration.
    unset($tentity, $tbundle, $vocab_reference, $source, $target);

    $source_terms = $entity->$source_field->referencedEntities();
    // We want a 'pool' of referenced terms and their ancestors to go through.
    $source_pool = $source_terms;
    foreach ($source_terms as $source_term) {
      /** @var \Drupal\taxonomy\TermInterface $source_term */
      /** @var \Drupal\taxonomy\TermStorageInterface $term_storage */
      $term_storage = \Drupal::entityTypeManager()->getStorage('taxonomy_term');
      $parents = $term_storage->loadAllParents($source_term->id());
      $source_pool += $parents;
    }
    unset($source_term, $source_terms);

    $target_tids = [];
    foreach ($source_pool as $source_term) {
      $tid = $source_term->$filter_field->target_id ?? NULL;
      if ($tid) {
        $target_tids[$tid] = $tid;
      }
    }
    $entity->set($target_field, $target_tids, FALSE);
  }
}